import sys
import os

# 添加项目的根目录到 sys.path
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import sentry_sdk
import uvicorn
import pandas as pd
import asyncio
from fastapi import FastAPI
from datetime import datetime


from starlette.middleware.cors import CORSMiddleware

from src.exceptions import register_exception_handler
from src.tortoises import register_tortoise_orm
from src.xtdata.router import router as xtdata_router
from src.backtest.router import router as backtest_router
from src.combination.router import router as combine_router
from src.models.financial_reports import FinancialReport 
from src.utils.update_financial_reports_spider import combined_search_and_list
from src.financial_reports.router import financial_reports_router
from src.utils.generate_pinyin_abbreviation import generate_pinyin_abbreviation

from xtquant import xtdata
from src.settings.config import app_configs, settings

import adata
import akshare as ak


app = FastAPI(**app_configs)

register_tortoise_orm(app)

register_exception_handler(app)

app.include_router(xtdata_router, prefix="/getwancedata", tags=["数据接口"])
app.include_router(backtest_router, prefix="/backtest", tags=["回测接口"])
app.include_router(combine_router, prefix="/combine", tags=["组合接口"])
app.include_router(financial_reports_router, prefix="/financial-reports", tags=["财报接口"])

if settings.ENVIRONMENT.is_deployed:
    sentry_sdk.init(
        dsn=settings.SENTRY_DSN,
        environment=settings.ENVIRONMENT,
    )

app.add_middleware(
    CORSMiddleware,
    allow_origins=settings.CORS_ORIGINS,
    allow_origin_regex=settings.CORS_ORIGINS_REGEX,
    allow_credentials=True,
    allow_methods=("GET", "POST", "PUT", "PATCH", "DELETE", "OPTIONS"),
    allow_headers=settings.CORS_HEADERS,
)


@app.get("/")
async def root():
    return {"message": "Hello, FastAPI!"}

# 定时检查和数据抓取函数
async def run_data_fetcher():
    print("财报抓取启动")
    while True:
        # 获取数据库中最新记录的时间
        latest_record = await FinancialReport .all().order_by("-created_at").first()
        latest_db_date = latest_record.created_at if latest_record else pd.Timestamp("1970-01-01")

        # 将最新数据库日期设为无时区，以便比较
        latest_db_date = latest_db_date.replace(tzinfo=None)

        # 当前时间
        current_time = pd.Timestamp(datetime.now()).replace(tzinfo=None)

       # 检查当前时间是否超过数据库最新记录时间 12 个小时
        if (current_time - latest_db_date).total_seconds() > 43200: 
            print("启动财报数据抓取...")
            await combined_search_and_list()
        else:
            print("未满足条件，跳过抓取任务。")

        # 休眠 12 小时（43200 秒），然后再次检查
        await asyncio.sleep(43200)

async def test_liang_hua_ku():
    print("量化库测试函数启动")
   

# 启动时立即运行检查任务
@app.on_event("startup")
async def lifespan():
     # 启动时将任务放入后台，不阻塞启动流程
    asyncio.create_task(test_liang_hua_ku())
    asyncio.create_task(run_data_fetcher())
    


if __name__ == "__main__":
    uvicorn.run('src.main:app', host="0.0.0.0", port=8012, reload=True)
